package main

import (
	"fmt"

	"github.com/urfave/cli/v2"
	"github.com/urfave/cli/v2/altsrc"
)

var (
	configFileFlag = &cli.StringFlag{
		Name:    "config",
		Aliases: []string{"c"},
		Usage:   "The path to the config file",
	}

	grpcListenAddrFlag = &cli.StringFlag{
		Name:  "grpc-listen",
		Usage: "The listen address of the grpc server",
		Value: "0.0.0.0:9430",
	}

	rpcListenAddrFlag = &cli.StringFlag{
		Name:  "rpc-listen",
		Usage: "The listen address of the jsonrpc server",
		Value: "0.0.0.0:9431",
	}

	rpcHostFlag = &cli.StringFlag{
		Name:  "chain-rpc",
		Usage: "The host of the chain json rpc server",
	}

	storeContractFlag = &cli.StringFlag{
		Name:  "store-contract",
		Usage: "The address of the store contract",
	}

	rewardContractFlag = &cli.StringFlag{
		Name:  "reward-contract",
		Usage: "The address of the reward contract",
	}
)

func loadFlagsFromConfig(cliCtx *cli.Context, flags []cli.Flag) error {
	if cliCtx.IsSet(configFileFlag.Name) {
		return altsrc.InitInputSourceWithContext(
			flags,
			altsrc.NewTomlSourceFromFlagFunc(configFileFlag.Name),
		)(cliCtx)
	}
	return nil
}

// WrapFlags so that they can be loaded from alternative sources.
func wrapFlags(flags []cli.Flag) []cli.Flag {
	wrapped := make([]cli.Flag, 0, len(flags))
	for _, f := range flags {
		switch t := f.(type) {
		case *cli.BoolFlag:
			f = altsrc.NewBoolFlag(t)
		case *cli.DurationFlag:
			f = altsrc.NewDurationFlag(t)
		case *cli.GenericFlag:
			f = altsrc.NewGenericFlag(t)
		case *cli.Float64Flag:
			f = altsrc.NewFloat64Flag(t)
		case *cli.IntFlag:
			f = altsrc.NewIntFlag(t)
		case *cli.StringFlag:
			f = altsrc.NewStringFlag(t)
		case *cli.StringSliceFlag:
			f = altsrc.NewStringSliceFlag(t)
		case *cli.Uint64Flag:
			f = altsrc.NewUint64Flag(t)
		case *cli.UintFlag:
			f = altsrc.NewUintFlag(t)
		case *cli.PathFlag:
			f = altsrc.NewPathFlag(t)
		case *cli.Int64Flag:
			// Int64Flag does not work. See https://github.com/prysmaticlabs/prysm/issues/6478
			panic(fmt.Sprintf("unsupported flag type type %T", f))
		case *cli.IntSliceFlag:
			f = altsrc.NewIntSliceFlag(t)
		default:
			panic(fmt.Sprintf("cannot convert type %T", f))
		}
		wrapped = append(wrapped, f)
	}
	return wrapped
}
